/** Rill v0.0.454 https://www.npmjs.com/package/rill License: MIT*/
var Application, Request, Response, co, compose, http, parseDates, pathToReg, respond;

co = require("co");

pathToReg = require("path-to-regexp");

compose = require("./util/compose");

parseDates = require("./util/parseDates");

Request = require("./request");

Response = require("./response");

respond = require("./util/response");

http = (require("./hijackBrowser"));

Application = (function() {
  var j, len, method, ref;

  function Application() {
    if (!(this instanceof Application)) {
      return new Application();
    }
    this.middleware = [];
  }

  Application.prototype.listen = function() {
    var server;
    server = http.createServer(this.callback());
    return server.listen.apply(server, arguments);
  };

  Application.prototype.callback = function() {
    var fn;
    fn = co.wrap(compose([respond].concat(this.middleware)));
    return function(req, res) {
      if (res == null) {
        res = {};
      }
      return fn.call({}, new Request(req), new Response(res))["catch"](function(err) {
        return console.error(err.stack || err);
      });
    };
  };

  Application.prototype.use = function(middleware) {
    this.middleware.push(middleware);
    return this;
  };

  Application.prototype.mount = function(path, application) {
    var middlewares;
    if (!(application instanceof Application)) {
      throw new Error("Can only mount other Rill applicaitons.");
    }
    path = pathToReg(path, [], {
      end: false
    });
    middlewares = compose(application.middleware);
    this.use(regeneratorRuntime.mark(function callee$2$0(req, res, next) {
      return regeneratorRuntime.wrap(function callee$2$0$(context$3$0) {
        while (1) switch (context$3$0.prev = context$3$0.next) {
        case 0:
          if (!path.test(req.url.pathname)) {
            context$3$0.next = 6;
            break;
          }

          context$3$0.next = 3;
          return middlewares.call(this, new Request(req.originalReq, path), new Response(res.originalRes), next);
        case 3:
          return context$3$0.abrupt("return", context$3$0.sent);
        case 6:
          context$3$0.next = 8;
          return next;
        case 8:
          return context$3$0.abrupt("return", context$3$0.sent);
        case 9:
        case "end":
          return context$3$0.stop();
        }
      }, callee$2$0, this);
    }));
    return this;
  };

  ref = ["options", "head", "get", "put", "post", "delete"];
  for (j = 0, len = ref.length; j < len; j++) {
    method = ref[j];
    Application.prototype[method] = (function(method) {
      return function(path, middleware) {
        var keys;
        keys = [];
        path = pathToReg(path, keys);
        method = method.toUpperCase();
        this.use(regeneratorRuntime.mark(function callee$3$0(req, res, next) {
          var i, k, len1, match, name, optional, ref1, val;

          return regeneratorRuntime.wrap(function callee$3$0$(context$4$0) {
            while (1) switch (context$4$0.prev = context$4$0.next) {
            case 0:
              if (!(req.method === method && (match = req.url.pathname.match(path)))) {
                context$4$0.next = 19;
                break;
              }

              req.params = {};
              i = k = 0, len1 = keys.length;
            case 3:
              if (!(k < len1)) {
                context$4$0.next = 14;
                break;
              }

              ref1 = keys[i], name = ref1.name, optional = ref1.optional;
              val = match[i + 1];

              if (optional || (val != null)) {
                context$4$0.next = 10;
                break;
              }

              context$4$0.next = 9;
              return next;
            case 9:
              return context$4$0.abrupt("return", context$4$0.sent);
            case 10:
              req.params[name] = val;
            case 11:
              i = ++k;
              context$4$0.next = 3;
              break;
            case 14:
              context$4$0.next = 16;
              return middleware.call(this, req, res, next);
            case 16:
              return context$4$0.abrupt("return", context$4$0.sent);
            case 19:
              context$4$0.next = 21;
              return next;
            case 21:
              return context$4$0.abrupt("return", context$4$0.sent);
            case 22:
            case "end":
              return context$4$0.stop();
            }
          }, callee$3$0, this);
        }));
        return this;
      };
    })(method);
  }

  return Application;

})();

module.exports = Application;
