/** Rill v0.0.454 https://www.npmjs.com/package/rill License: MIT*/
var Application, Request, Response, co, compose, http, parseDates, pathToReg, respond;

co = require("co");

pathToReg = require("path-to-regexp");

compose = require("./util/compose");

parseDates = require("./util/parseDates");

Request = require("./request");

Response = require("./response");

respond = require("./util/response");

http = (http = require("http"));

Application = (function() {
  var j, len, method, ref;

  function Application() {
    if (!(this instanceof Application)) {
      return new Application();
    }
    this.middleware = [];
  }

  Application.prototype.listen = function() {
    var server;
    server = http.createServer(this.callback());
    return server.listen.apply(server, arguments);
  };

  Application.prototype.callback = function() {
    var fn;
    fn = co.wrap(compose([respond].concat(this.middleware)));
    return function(req, res) {
      if (res == null) {
        res = {};
      }
      return fn.call({}, new Request(req), new Response(res))["catch"](function(err) {
        return console.error(err.stack || err);
      });
    };
  };

  Application.prototype.use = function(middleware) {
    this.middleware.push(middleware);
    return this;
  };

  Application.prototype.mount = function(path, application) {
    var middlewares;
    if (!(application instanceof Application)) {
      throw new Error("Can only mount other Rill applicaitons.");
    }
    path = pathToReg(path, [], {
      end: false
    });
    middlewares = compose(application.middleware);
    this.use(function*(req, res, next) {
      if (path.test(req.url.pathname)) {
        return (yield middlewares.call(this, new Request(req.originalReq, path), new Response(res.originalRes), next));
      } else {
        return (yield next);
      }
    });
    return this;
  };

  ref = ["options", "head", "get", "put", "post", "delete"];
  for (j = 0, len = ref.length; j < len; j++) {
    method = ref[j];
    Application.prototype[method] = (function(method) {
      return function(path, middleware) {
        var keys;
        keys = [];
        path = pathToReg(path, keys);
        method = method.toUpperCase();
        this.use(function*(req, res, next) {
          var i, k, len1, match, name, optional, ref1, val;
          if ((req.method === method) && (match = req.url.pathname.match(path))) {
            req.params = {};
            for (i = k = 0, len1 = keys.length; k < len1; i = ++k) {
              ref1 = keys[i], name = ref1.name, optional = ref1.optional;
              val = match[i + 1];
              if (!(optional || (val != null))) {
                return (yield next);
              }
              req.params[name] = val;
            }
            return (yield middleware.call(this, req, res, next));
          } else {
            return (yield next);
          }
        });
        return this;
      };
    })(method);
  }

  return Application;

})();

module.exports = Application;
