co         = require("co")
pathToReg  = require("path-to-regexp")
compose    = require("./util/compose")
parseDates = require("./util/parseDates")
Request    = require("./request")
Response   = require("./response")
respond    = require("./util/response")
http       = (
	if process.browser then require("./hijackBrowser")
	else http = require("http")
)


class Application
	constructor: ->
		return new Application() unless this instanceof Application
		@middleware = []

	listen: ->
		server = http.createServer(@callback())
		server.listen.apply(server, arguments)

	callback: ->
		fn = co.wrap(compose([respond].concat(@middleware)))
		(req, res = {})->
			fn.call({}, new Request(req), new Response(res)).catch((err)->
				console.error(err.stack or err)
			)

	use: (middleware)->
		@middleware.push(middleware)
		return this

	mount: (path, application)->
		unless application instanceof Application
			throw new Error("Can only mount other Rill applicaitons.")

		path = pathToReg(path, [], end: no)
		middlewares = compose(application.middleware)
		@use((req, res, next)->
			if path.test(req.url.pathname)
				yield middlewares.call(@,
					new Request(req.originalReq, path),
					new Response(res.originalRes),
					next
				)
			else yield next
		)

		return this

	for method in ["options", "head", "get", "put", "post", "delete"]
		@::[method] = do (method)-> (path, middleware)->
			keys = []
			path = pathToReg(path, keys)
			method = method.toUpperCase()

			@use((req, res, next)->
				if (req.method is method) and match = req.url.pathname.match(path)
					req.params = {}
					for { name, optional }, i in keys
						val = match[i + 1]
						return yield next unless optional or val?
						req.params[name] = val
					yield middleware.call(@, req, res, next)
				else yield next
			)
			return this


module.exports = Application
